@@ -46,7 +46,7 @@
  ***/
 
 /* 
- * Find location of special windows diretories.
+ * Find location of special windows directories.
  * Copied from glib sources since they have declared it static.
  * Windows only.
  * CHECKME: Is it okay to return UTF8?
@@ -105,6 +105,66 @@ static char* get_exe_path()
 }
 #endif
 
+/*
+ * Create / compare signatures for our installation directory
+ */
+
+#ifdef SYS_MINGW
+static unsigned char *create_signature()
+{  struct MD5Context md5ctxt;
+   unsigned char *digest = g_malloc(20);
+   char buf[80];
+
+   sprintf(buf,"dvdisaster %s signature string", Closure->cookedVersion);
+
+   MD5Init(&md5ctxt);
+   MD5Update(&md5ctxt, buf, strlen(buf));
+   MD5Final(digest+2, &md5ctxt);
+   digest[ 0] = digest[ 7]+19;
+   digest[ 1] = digest[ 9]+58;
+   digest[18] = digest[12]+31;
+   digest[19] = digest[15]+5;
+
+   return digest;
+}
+
+void WriteSignature()
+{  char loc[strlen(Closure->binDir) + strlen("\\signature")+ 10];
+   unsigned char *sig = create_signature();
+   FILE *file;
+
+   /* processing of error conditions not necessary */
+
+   sprintf(loc, "%s\\signature", Closure->binDir);
+   if(!(file = fopen(loc, "wb")))
+     return;
+   fwrite(sig, 20, 1, file);
+   fclose(file);
+   g_free(sig);
+}
+
+int VerifySignature()
+{  char loc[strlen(Closure->binDir) + strlen("\\signature")+ 10];
+   unsigned char *sig = create_signature();
+   char buf[20];
+   FILE *file;
+   int result;
+
+   sprintf(loc, "%s\\signature", Closure->binDir);
+   if(!(file = fopen(loc, "rb")))
+     return FALSE;
+
+   fread(buf, 20, 1, file);
+   fclose(file);
+
+   result = !memcmp(sig, buf, 20);
+
+   return result;
+}
+
+#endif
+
+
 static void get_base_dirs()
 {  struct stat mystat;
 #ifdef WITH_LOGFILE_YES
@@ -349,14 +409,14 @@ static void update_dotfile()
    g_fprintf(dotfile, "adaptive-read:   %d\n", Closure->adaptiveRead);
    g_fprintf(dotfile, "auto-suffix:     %d\n", Closure->autoSuffix);
    g_fprintf(dotfile, "cache-size:      %d\n", Closure->cacheMB);
-   g_fprintf(dotfile, "cd-size:         %lld\n", Closure->cdSize);
+   g_fprintf(dotfile, "cd-size:         %lld\n", (long long int)Closure->cdSize);
    g_fprintf(dotfile, "dao:             %d\n", Closure->noTruncate);
    g_fprintf(dotfile, "dotfile-version: %d\n", Closure->dotFileVersion);
-   g_fprintf(dotfile, "dvd-size1:       %lld\n", Closure->dvdSize1);
-   g_fprintf(dotfile, "dvd-size2:       %lld\n", Closure->dvdSize2);
+   g_fprintf(dotfile, "dvd-size1:       %lld\n", (long long int)Closure->dvdSize1);
+   g_fprintf(dotfile, "dvd-size2:       %lld\n", (long long int)Closure->dvdSize2);
    g_fprintf(dotfile, "fill-unreadable: %d\n", Closure->fillUnreadable);
    g_fprintf(dotfile, "jump:            %d\n", Closure->sectorSkip);
-   g_fprintf(dotfile, "medium-size:     %lld\n", Closure->mediumSize);
+   g_fprintf(dotfile, "medium-size:     %lld\n", (long long int)Closure->mediumSize);
    g_fprintf(dotfile, "method-name:     %s\n", Closure->methodName);
    g_fprintf(dotfile, "query-size:      %d\n", Closure->querySize);
    g_fprintf(dotfile, "read-and-create: %d\n", Closure->readAndCreate);
@@ -530,6 +590,7 @@ void FreeClosure()
 
    cond_free(Closure->methodName);
    cond_free(Closure->dotFile);
+   cond_free(Closure->logFile);
    cond_free(Closure->binDir);
    cond_free(Closure->docDir);
    cond_free(Closure->browser);
