@@ -124,10 +124,12 @@ typedef enum
    MODE_BYTESET, 
    MODE_ERASE, 
    MODE_LIST_ASPI,
+   MODE_MARKED_IMAGE,
    MODE_RANDOM_ERR, 
    MODE_RANDOM_IMAGE,
    MODE_SEND_CDB,
    MODE_SHOW_SECTOR, 
+   MODE_SIGN,
    MODE_TRUNCATE,
    MODE_ZERO_UNREADABLE,
 
@@ -139,7 +141,6 @@ typedef enum
    MODIFIER_DAO, 
    MODIFIER_DEBUG,
    MODIFIER_FILL_UNREADABLE,
-   MODIFIER_KEEP_STYLE,
    MODIFIER_QUERY_SIZE,
    MODIFIER_RANDOM_SEED,
    MODIFIER_SIMULATE_DEFECTS,
@@ -327,10 +328,10 @@ int main(int argc, char *argv[])
 	{"help", 0, 0, 'h'},
         {"image", 1, 0, 'i'},
 	{"jump", 1, 0, 'j'},
-	{"keep-style", 0, 0, MODIFIER_KEEP_STYLE },
 #ifdef SYS_MINGW
 	{"list", 0, 0, 'l' },
 #endif
+	{"marked-image", 1, 0, MODE_MARKED_IMAGE },
 	{"method", 2, 0, 'm' },
 	{"query-size", 1, 0, MODIFIER_QUERY_SIZE },
         {"prefix", 1, 0, 'p'},
@@ -342,6 +343,7 @@ int main(int argc, char *argv[])
 	{"scan", 2, 0,'s'},
 	{"send-cdb", 1, 0, MODE_SEND_CDB},
 	{"show-sector", 1, 0, MODE_SHOW_SECTOR},
+	{"sign", 0, 0, MODE_SIGN},
 	{"sim-defects", 1, 0, MODIFIER_SIMULATE_DEFECTS},
 	{"speed-warning", 2, 0, MODIFIER_SPEED_WARNING},
 	{"spinup-delay", 1, 0, MODIFIER_SPINUP_DELAY},
@@ -391,7 +393,13 @@ int main(int argc, char *argv[])
 	           break;
          case 'n': if(optarg) 
 		   {  Closure->redundancy = g_strdup(optarg); 
-		      Closure->mediumSize = (gint64)atoll(optarg);
+		      if(!strcmp(optarg, "CD") || !strcmp(optarg, "cd"))
+			   Closure->mediumSize = CDR_SIZE;
+		      else if(!strcmp(optarg, "DVD") || !strcmp(optarg, "dvd"))
+			   Closure->mediumSize = DVD_SL_SIZE;
+		      else if(!strcmp(optarg, "DVD9") || !strcmp(optarg, "dvd9"))
+			   Closure->mediumSize = DVD_DL_SIZE;
+		      else Closure->mediumSize = (gint64)atoll(optarg);
 		      break;
 		   }
          case 'e': if(optarg) 
@@ -444,9 +452,6 @@ int main(int argc, char *argv[])
 	   }
 	   else Closure->truncate = 1; /* truncate confirmation for fix mode */
 	   break;
-         case MODIFIER_KEEP_STYLE:
-	   Closure->keepStyle = 1;
-	   break;
          case MODIFIER_DEBUG:
 	   Closure->debugMode = TRUE;
 	   break;
@@ -484,14 +489,18 @@ int main(int argc, char *argv[])
 	   FreeClosure();
 	   exit(EXIT_SUCCESS); 
 	   break;
-         case MODE_SEND_CDB: 
-	   mode = MODE_SEND_CDB;
+         case MODE_BYTESET:
+	   mode = MODE_BYTESET;
 	   debug_arg = g_strdup(optarg);
 	   break;
          case MODE_ERASE: 
 	   mode = MODE_ERASE;
 	   debug_arg = g_strdup(optarg);
 	   break;
+         case MODE_MARKED_IMAGE:
+	   mode = MODE_MARKED_IMAGE;
+	   debug_arg = g_strdup(optarg);
+	   break;
          case MODE_RANDOM_ERR:
 	   mode = MODE_RANDOM_ERR;
 	   debug_arg = g_strdup(optarg);
@@ -500,10 +509,13 @@ int main(int argc, char *argv[])
 	   mode = MODE_RANDOM_IMAGE;
 	   debug_arg = g_strdup(optarg);
 	   break;
-         case MODE_BYTESET:
-	   mode = MODE_BYTESET;
+         case MODE_SEND_CDB: 
+	   mode = MODE_SEND_CDB;
 	   debug_arg = g_strdup(optarg);
 	   break;
+         case MODE_SIGN:
+	   mode = MODE_SIGN;
+	   break;
          case MODE_SHOW_SECTOR:
 	   mode = MODE_SHOW_SECTOR;
 	   debug_arg = g_strdup(optarg);
@@ -524,8 +536,10 @@ int main(int argc, char *argv[])
         case MODE_ERASE:
         case MODE_SEND_CDB:
         case MODE_SHOW_SECTOR:
+        case MODE_MARKED_IMAGE:
         case MODE_RANDOM_ERR:
         case MODE_RANDOM_IMAGE:
+        case MODE_SIGN:
         case MODE_TRUNCATE:
         case MODE_ZERO_UNREADABLE:
 	  mode = MODE_HELP;
@@ -534,6 +548,18 @@ int main(int argc, char *argv[])
 	  
    PrepareDeadSector();
 
+#ifdef WIN_CONSOLE
+   if(mode != MODE_SIGN && !VerifySignature())
+   {  char version[80];
+
+      if(Closure->version % 100)
+           sprintf(version, "dvdisaster-%s.%d-setup.exe", VERSION, Closure->version%100);
+      else sprintf(version, "dvdisaster-%s-setup.exe", VERSION);
+      Stop(_("dvdisaster is not properly installed.\n"
+	     "Please execute the installer program (%s) again.\n"), version);
+   }
+#endif
+
    /*** Parse the sector ranges for --read and --scan */
 
    if(read_range)
@@ -561,8 +587,16 @@ int main(int argc, char *argv[])
    }
 
    /*** Dispatch action depending on mode.
-        The major modes can be executed in sequence 
-	(although not all combinations may be really useful) */
+        The major modes can be executed in sequence, 
+	but not all combinations may be really useful.
+
+	The GUI version for Windows does not have an open console,
+	so no command line actions can be carried out. 
+	Force opening the GUI by clearing the mode variable. */
+
+#if defined(SYS_MINGW) && !defined(WIN_CONSOLE)
+   mode = MODE_NONE;
+#endif
 
    switch(mode)
    {  case MODE_SEQUENCE:
@@ -607,8 +641,12 @@ int main(int argc, char *argv[])
 	 RandomError(Closure->imageName, debug_arg);
 	 break;
 
+      case MODE_MARKED_IMAGE:
+	 RandomImage(Closure->imageName, debug_arg, 1);
+	 break;
+
       case MODE_RANDOM_IMAGE:
-	 RandomImage(Closure->imageName, debug_arg);
+	RandomImage(Closure->imageName, debug_arg, 0);
 	 break;
 
       case MODE_TRUNCATE:
@@ -623,6 +661,11 @@ int main(int argc, char *argv[])
       case MODE_LIST_ASPI:
 	 ListAspiDrives();
 	 break;
+
+      case MODE_SIGN:
+	 WriteSignature();
+	 exit(0);
+	 break;
 #endif
       default:
 	break;
@@ -632,7 +675,11 @@ int main(int argc, char *argv[])
 
    /*** If no mode was selected, print the help screen. */
 
+#ifdef WIN_CONSOLE
+   if(mode == MODE_HELP || mode == MODE_NONE)
+#else
    if(mode == MODE_HELP)
+#endif
    {  
      /* TRANSLATORS: Program options like -r and --read are not to be translated
 	to avoid confusion when discussing the program in international forums. */
@@ -674,15 +721,13 @@ int main(int argc, char *argv[])
 	     "  --spinup-delay n       - wait n seconds for drive to spin up\n"
 	     "  --split-files          - split image into files <= 2GB\n\n"));
 
-      PrintCLI(_("Graphical user interface options\n"
-	     "  --keep-style           - do not override the style settings\n\n"));
-
       if(Closure->debugMode)
       { PrintCLI(_("Debugging options (purposefully undocumented and possibly harmful)\n"
 	     "  --debug           - enables the following options\n"
 	     "  --byteset s,i,b   - set byte i in sector s to b\n"
 	     "  --erase sector    - erase the given sector\n"
 	     "  --erase n-m       - erase sectors n - m, inclusively\n"
+	     "  --marked-image n  - create image with n marked random sectors\n"
 	     "  --random-errors r,e seed image with (correctable) random errors\n"
 	     "  --random-image n  - create image with n sectors of random numbers\n"
 	     "  --random-seed n   - random seed for built-in random number generator\n"
@@ -692,11 +737,20 @@ int main(int argc, char *argv[])
 	     "  --truncate n      - truncates image to n sectors\n"
 	     "  --zero-unreadable - replace the \"unreadable sector\" markers with zeros\n\n"));
       }
+
+#ifdef WIN_CONSOLE
+      PrintCLI(_("NOTE: This is the Windows console version of dvdisaster.\n"
+		 "The version providing a graphical user interface is called\n"
+		 "dvdisaster-win.exe (also contained in this installation).\n\n"));
+#endif
       FreeClosure();
       exit(EXIT_FAILURE);
    }
 
-   /* If no mode was selected at the command line, start the graphical user interface. */
+   /* If no mode was selected at the command line, 
+      start the graphical user interface. 
+      Unless we are in Windows console mode where starting the GUI
+      won't work. */
 
    if(mode == MODE_NONE)
    {  
@@ -704,7 +758,7 @@ int main(int argc, char *argv[])
       ReadDotfile();
       CreateMainWindow(&argc, &argv);
    }
-   
+
    FreeClosure();
 
    exit(exitCode);
