@@ -178,7 +178,6 @@ typedef struct _GlobalClosure
    /*** GUI-related things */
 
    int guiMode;         /* TRUE if GUI is active */
-   int keepStyle;       /* TRUE = don't tinker with style */
    int stopActions;     /* crude method to stop ongoing action(s) */
 
    GtkWidget *logWidget;     /* Dialog for the log display */
@@ -369,6 +368,8 @@ void InitClosure(void);
 void ClearCrcCache(void);
 void FreeClosure(void);
 void ReadDotfile(void);
+void WriteSignature(void);
+int  VerifySignature(void);
 
 /***
  *** crc32.c
@@ -421,7 +422,7 @@ void RedrawCurve(Curve*, int, int);
 void Byteset(char*);
 void Erase(char*);
 void RandomError(char*, char*);
-void RandomImage(char*, char*);
+void RandomImage(char*, char*, int);
 void SendCDB(char*);
 void ShowSector(char*);
 Bitmap* SimulateDefects(gint64);
@@ -699,6 +700,8 @@ void UpdateMethodPreferences(void);
 void HidePreferences(void);
 void FreePreferences(void*);
 
+void UpdatePrefsQuerySize(void);
+
 /***
  *** print_sense.c
  ***/
@@ -746,6 +749,7 @@ void ReadMediumAdaptive(gpointer);
 #define ADAPTIVE_READ_SPIRAL_SIZE 4800
 
 void ResetAdaptiveReadWindow();
+void SetAdaptiveReadMinimumPercentage(int);
 void CreateAdaptiveReadWindow(GtkWidget*);
 
 void ClipReadAdaptiveSpiral(int);
@@ -807,12 +811,6 @@ void DrawSpiralSegment(Spiral*, GdkColor*, int);
 void DrawSpiralLabel(Spiral*, PangoLayout*, char*, GdkColor*, int, int);
 void MoveSpiralCursor(Spiral*, int);
 
-/***
- *** style.c
- ***/
-
-void AdjustStyle();
-
 /***
  *** welcome-window.c
  ***/
