@@ -1,7 +1,7 @@
 #ifndef MD5_H
 #define MD5_H
 
-#ifdef TILEPACK
+#if defined(PNGPACK) || defined(SIMPLE_MD5SUM)
   typedef unsigned int guint32;
 #endif
 
@@ -16,4 +16,6 @@ void MD5Update(struct MD5Context *context, unsigned char const *buf,
 			unsigned len);
 void MD5Final(unsigned char digest[16], struct MD5Context *context);
 
+void AsciiDigest(char*, unsigned char*);
+
 #endif /* MD5_H */
