@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004,2005 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2006 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -68,9 +68,17 @@ static void menu_cb(GtkWidget *widget, gpointer data)
    
 	g_free(Closure->imageName);
 	Closure->imageName = g_strdup(gtk_entry_get_text(GTK_ENTRY(Closure->imageEntry)));
+	if(!Closure->imageName || !strlen(Closure->imageName))
+	{  if(Closure->imageName) g_free(Closure->imageName);
+	   Closure->imageName = g_strdup("none");
+	}
 
 	g_free(Closure->eccName);
 	Closure->eccName = g_strdup(gtk_entry_get_text(GTK_ENTRY(Closure->eccEntry)));
+	if(!Closure->eccName || !strlen(Closure->eccName))
+	{  if(Closure->eccName) g_free(Closure->eccName);
+	   Closure->eccName = g_strdup("none");
+	}
 
 	/* and quit */
 
@@ -118,7 +126,6 @@ static void menu_cb(GtkWidget *widget, gpointer data)
         g_print("Menu/Toolbar action %d\n",GPOINTER_TO_INT(data));
         break;
   }
-
 }
 
 /***
@@ -179,7 +186,7 @@ GtkWidget *CreateMenuBar(GtkWidget *parent)
    add_menu_button(menu_strip, _("menu|Select Parity File"), MENU_FILE_ECC);
    add_menu_button(menu_strip, _("menu|Quit"), MENU_FILE_QUIT);
 
-   menu_anchor = gtk_menu_item_new_with_label(_("menu|File"));
+   menu_anchor = gtk_menu_item_new_with_label(_utf("menu|File"));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_anchor), menu_strip);
    gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_anchor);
 
@@ -200,7 +207,7 @@ GtkWidget *CreateMenuBar(GtkWidget *parent)
    add_menu_button(menu_strip, _("menu|Change log"), MENU_HELP_CHANGELOG);
    add_menu_button(menu_strip, _("menu|To do list"), MENU_HELP_TODO);
 
-   menu_anchor = gtk_menu_item_new_with_label(_("menu|Help"));
+   menu_anchor = gtk_menu_item_new_with_label(_utf("menu|Help"));
    gtk_menu_item_right_justify(GTK_MENU_ITEM(menu_anchor));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_anchor), menu_strip);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_anchor);
@@ -308,7 +315,7 @@ static void file_select_cb(GtkWidget *widget, gpointer data)
 	 g_free(Closure->imageName);
 	 Closure->imageName = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(Closure->imageFileSel)));
 	 if(Closure->autoSuffix)
-	   Closure->imageName = ApplyAutoSuffix(Closure->imageName, "img");
+	   Closure->imageName = ApplyAutoSuffix(Closure->imageName, "iso");
 	 gtk_entry_set_text(GTK_ENTRY(Closure->imageEntry), Closure->imageName);
 	 gtk_editable_set_position(GTK_EDITABLE(Closure->imageEntry), -1);
 	 gtk_widget_hide(Closure->imageFileSel);
@@ -365,15 +372,20 @@ static void file_select_cb(GtkWidget *widget, gpointer data)
 
 void set_path(GtkWidget *entry, char *path)
 {
-   if(path[0] == '/' || path[1] == ':')
+  if(path[0] == '/' || path[0] == '\\' || path[1] == ':' || strlen(path) < 1)
    {  gtk_entry_set_text(GTK_ENTRY(entry), path);
       gtk_editable_set_position(GTK_EDITABLE(entry), -1);
    }
    else
-     {  char buf[PATH_MAX + strlen(path) + 2];
+   {  char buf[PATH_MAX + strlen(path) + 2];
 
+#ifdef SYS_MINGW
+      getcwd(buf, PATH_MAX);
+      strcat(buf,"\\");
+#else
       getcwd(buf, PATH_MAX);
       strcat(buf,"/");
+#endif
       strcat(buf,path);
       gtk_entry_set_text(GTK_ENTRY(entry), buf);
       gtk_editable_set_position(GTK_EDITABLE(entry), -1);
@@ -392,7 +404,7 @@ static void suffix_cb(GtkWidget *widget, gpointer data)
 
    if(!ecc_file)
    {  Closure->imageName = g_strdup(gtk_entry_get_text(GTK_ENTRY(Closure->imageEntry)));
-      Closure->imageName = ApplyAutoSuffix(Closure->imageName, "img");
+      Closure->imageName = ApplyAutoSuffix(Closure->imageName, "iso");
       gtk_entry_set_text(GTK_ENTRY(Closure->imageEntry), Closure->imageName);
    }
    else
@@ -447,7 +459,7 @@ GtkWidget *CreateToolBar(GtkWidget *parent)
    }
 
    if(!Closure->deviceNodes->len)
-   {   gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _("No drives found"));
+   {   gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _utf("No drives found"));
    }
 
    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), dev_idx);
@@ -471,7 +483,7 @@ GtkWidget *CreateToolBar(GtkWidget *parent)
    }
 
    if(!Closure->deviceNodes->len)
-   {  GtkWidget *item = gtk_menu_item_new_with_label(_("No drives found"));
+   {  GtkWidget *item = gtk_menu_item_new_with_label(_utf("No drives found"));
       gtk_menu_shell_append(GTK_MENU_SHELL(menu_strip), item);
    }
 
