@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004,2005 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2006 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -21,31 +21,6 @@
 
 #include "dvdisaster.h"
 
-/***
- *** Windows utilities
- ***/
-
-/*
- * Adapt the Windows approach to large files
- * to the POSIX way.
- */
-
-#ifdef SYS_MINGW
-
-#include <windows.h>
-
-int fseeko(FILE *file, gint64 pos, int mode)
-{  
-   if(mode != SEEK_SET) 
-   {  Stop(_("fseeko mode must be SEEK_SET"));
-      return -1;
-   }
-
-   return fsetpos(file, &pos);
-}
-
-#endif
-
 /***
  *** gettext() convenience
  ***/
@@ -115,8 +90,9 @@ char *sgettext_utf8(char *msgid)
 
 /*
  * Convert between 8 bytes of unsigned char and gint64.
- * Mainly needed as various version of gcc used different alignment for gint64 values
- * so that we can not use gint64 directly in the EccHeader structure.
+ * Needed since the EccHeader contains misaligned 64bit values
+ * for historical reasons.
+ * The uchar version uses a little endian representation.
  */
 
 gint64 uchar_to_gint64(unsigned char *bytes)
@@ -124,30 +100,22 @@ gint64 uchar_to_gint64(unsigned char *bytes)
 
    memcpy(&out, bytes, sizeof(gint64));
 
+#ifdef HAVE_BIG_ENDIAN
+   return SwapBytes64(out);
+#else
    return out;
+#endif
 }
 
 void gint64_to_uchar(unsigned char *out, gint64 in)
 {
+#ifdef HAVE_BIG_ENDIAN
+  in = SwapBytes64(in);
+#endif
+
    memcpy(out, &in, sizeof(in));
 }
 
-/*
- * Convert md5 digest into printable format.
- */
-
-static char hextab[] = "0123456789abcdef";
-
-void AsciiDigest(char *out, unsigned char *in)
-{  int i,o;
-
-   for(i=0, o=0; i<16; i++)
-   {  out[o++] = hextab[in[i]>>4];
-      out[o++] = hextab[in[i]&0x0f];
-   }
-   out[o] = 0;
-}
-
 /*
  * Calculate total number of sectors (including the last sector)
  * and the remaining bytes in the last sector for a given medium size.
@@ -175,7 +143,7 @@ static void print_greetings(FILE *where)
    if(greetings_shown) return;
 
    greetings_shown = 1;
-   g_fprintf(where, _("dvdisaster-%s%sCopyright 2004,2005 Carsten Gnoerlich.\n"),
+   g_fprintf(where, _("dvdisaster-%s%sCopyright 2004-2006 Carsten Gnoerlich.\n"),
 	     VERSION, strstr(VERSION,"pl") ? " " : "  ");
    /* TRANSLATORS: Excluding all kinds of warranty might be harmful under your
       legislature. If in doubt, just translate the following like "This is free
@@ -222,7 +190,12 @@ void PrintProgress(char *format, ...)
    print_greetings(stderr);
 
    va_start(argp, format);
-   n = g_vsnprintf(msg, 256, format, argp);
+   g_vsnprintf(msg, 256, format, argp);
+#ifdef SYS_MINGW
+   n = strlen(msg);  /* UTF-8 is broken under Windows */
+#else
+   n = g_utf8_strlen(msg,-1);
+#endif
    va_end(argp);
 
    if(n>255) 
@@ -273,6 +246,109 @@ void PrintLog(char *format, ...)
    va_end(argp);
 }
 
+/*
+ * Same as PrintLog(), but does nothing unless Closure->verbose is true
+ */
+
+void Verbose(char *format, ...)
+{  va_list argp;
+
+   if(!Closure->verbose)
+    return;
+
+   va_start(argp, format);
+
+   if(Closure->guiMode)
+   {  char *tmp,*utf_tmp;
+
+      tmp = g_strdup_vprintf(format, argp);
+      utf_tmp = g_locale_to_utf8(tmp, -1, NULL, NULL, NULL);
+      g_string_append(Closure->logString, utf_tmp);
+
+      UpdateLog();
+   
+      g_free(tmp);
+      g_free(utf_tmp);
+   }
+   else 
+   {
+      print_greetings(stderr);
+      g_vfprintf(stderr, format, argp);
+
+      fflush(stderr);   /* at least needed for Windows */
+   }
+
+   va_end(argp);
+}
+
+/*
+ * Print a message to the log file.
+ * Tries hard to make the log messages survive a system crash.
+ */
+
+#ifdef WITH_LOGFILE_YES
+void PrintLogFile(char *format, ...)
+{  FILE *file;
+   va_list argp;
+
+   va_start(argp, format);
+
+   file = fopen(Closure->logFile, "a");
+   if(!file)
+     return;
+
+   g_vfprintf(file, format, argp);
+   fflush(file);
+   fclose(file);
+
+   va_end(argp);
+}
+#else
+void PrintLogFile(char *format, ...) { }
+#endif
+
+/*
+ * Print timing results to console and log window 
+ */
+
+void PrintTimeToLog(GTimer *timer, char *format, ...)
+{  va_list argp;
+   gulong ignore;
+   double elapsed = g_timer_elapsed(timer, &ignore); 
+   double seconds = fmod(elapsed,60.0);
+   int minutes = (int)fmod(elapsed / 60.0, 60.0);
+   int hours = (int)(elapsed / 3600.0);
+   char *tmp1,*tmp2,*utf_tmp;
+
+   if(!Closure->verbose)
+     return;
+
+   va_start(argp, format);
+
+   tmp1 = g_strdup_vprintf(format, argp);
+   tmp2 = g_strdup_printf("%02d:%02d:%04.1f %s", hours, minutes, seconds, tmp1);
+
+   if(Closure->guiMode)
+   { 
+      utf_tmp = g_locale_to_utf8(tmp2, -1, NULL, NULL, NULL);
+      g_string_append(Closure->logString, utf_tmp);
+
+      UpdateLog();
+   
+      g_free(utf_tmp);
+   }
+   else
+   {  g_fprintf(stderr, tmp2);
+
+      fflush(stderr);   /* at least needed for Windows */
+   }
+
+   va_end(argp);
+
+   g_free(tmp1);
+   g_free(tmp2);
+}
+
 /*
  * Print a message either to the console
  * or show it in the given label
@@ -346,7 +422,8 @@ static void vlog_warning(char *format, va_list argp)
       UpdateLog();
    }
    else
-   {  g_fprintf(stderr, "%s", str->str);
+   {  print_greetings(stderr);
+      g_fprintf(stderr, "%s", str->str);
       fflush(stderr);   /* at least needed for Windows */
    }
 
@@ -465,7 +542,10 @@ void RegisterCleanup(char *error_title, void (*cleanup)(gpointer), gpointer data
  ***/
 
 /*
- * Thread creation
+ * Thread creation.
+ * Linux and Windows seem to provide at least 2MB of stack size per thread,
+ * but FreeBSD has insanely small defaults. If this turns out to be too low,
+ * we need the currently commented out alternative.
  */
 
 GThread* CreateGThread(GThreadFunc f, gpointer data)
@@ -474,6 +554,11 @@ GThread* CreateGThread(GThreadFunc f, gpointer data)
 
    t = g_thread_create(f, data, FALSE, &err);
 
+#if 0
+   t = g_thread_create_full(f, data, MY_MINIMUM_STACKSIZE,
+			    FALSE, FALSE, G_THREAD_PRIORITY_NORMAL, &err);
+#endif
+
    if(!t) Stop("Could not create thread: %s\n",err->message);
 
    return t;
@@ -572,7 +657,7 @@ void ShowMessage(GtkWindow *parent, char *msg, GtkMessageType type)
 
    mi->msg    = g_strdup(msg);
    mi->type   = type;
-   mi->window = parent; //Closure->window, 
+   mi->window = parent;
 
    g_idle_add(message_idle_func, mi);
 }
@@ -619,7 +704,6 @@ typedef struct
 static gboolean label_idle_func(gpointer data)
 {  label_info *li = (label_info*)data;
 
-//gtk_label_set_text(li->label, li->text);
    gtk_label_set_markup(li->label, li->text);
 
    g_free(li->text);
@@ -878,17 +962,26 @@ void SwitchAndSetFootline(GtkWidget *notebook, int page, GtkWidget *label, char
  * Rearrange buttons to OK Cancel order
  * in file dialogs
  * 
- * Should we use gtk_dialog_set_alternative_button_order()
- * in gtk+2.6 instead?
+ * gtk_dialog_set_alternative_button_order()
+ * has been introduced since gtk+2.6,
+ * but seems to work only in 2.8.
  */
 
 void ReverseCancelOK(GtkDialog *dialog)
-{  GtkWidget *box, *button ;
+{  
+#if GTK_MINOR_VERSION <= 6
+   GtkWidget *box, *button ;
 
    box = dialog->action_area; 
    button = ((GtkBoxChild*)(g_list_first(GTK_BOX(box)->children)->data))->widget;
 
    gtk_box_reorder_child(GTK_BOX(box), button, 1);
+#else
+   gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog),
+					   GTK_RESPONSE_OK,
+					   GTK_RESPONSE_CANCEL,
+					   -1);
+#endif
 }
 
 /*
