@@ -39,11 +39,10 @@ typedef struct
  ***/
 
 typedef struct
-{  //char **jumpFormat;
-   char *formatLinear;
+{  char *formatLinear;
    char *formatAdaptive;
 
-  GtkWidget *radioDrive, *radioISO, *radioECC;
+   GtkWidget *radioDrive, *radioISO, *radioECC;
    GtkWidget *radioLinear, *radioAdaptive;
    GtkWidget *rangeToggle, *rangeLab;
    GtkWidget *rangeSpin1, *rangeSpin2;
@@ -124,6 +123,21 @@ static void close_cb(GtkWidget *widget, gpointer data)
   HidePreferences();
 }
 
+/***
+ *** Setting preferences from external functions
+ ***/
+
+void UpdatePrefsQuerySize(void)
+{  prefs_context *pc = (prefs_context*)Closure->prefsContext;
+
+   if(Closure->prefsContext)
+     switch(Closure->querySize)
+     {  case 0: gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pc->radioDrive), TRUE); break;
+        case 1: gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pc->radioISO), TRUE); break;
+        case 2: gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pc->radioECC), TRUE); break;
+     } 
+}
+
 /*
  * Actions used in the preferences
  */
