@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004,2005 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2006 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -534,6 +534,11 @@ static struct error_info additional[] =
   {0x73,0x04,R,"Program memory area update failure"},
   {0x73,0x05,R,"Program memory area is full"},
   {0x73,0x06,R,"RMA/PMA is full"},
+  /*
+   * Faked error by defect simulation mode
+   */
+  {0xff,0xff,R,"Simulated medium defect"},
+
   {0, 0, 0, NULL}
 };
 
