@@ -100,9 +100,9 @@ static void cleanup(gpointer data)
 static void insert_buttons(GtkDialog *dialog)
 {  
   gtk_dialog_add_buttons(dialog, 
-			 _("Ignore once"), 1,
-			 _("Ignore always"), 2,
-			 _("Abort"), 0, NULL);
+			 _utf("Ignore once"), 1,
+			 _utf("Ignore always"), 2,
+			 _utf("Abort"), 0, NULL);
 } 
 
 void ReadMediumLinear(gpointer data)
