@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004,2005 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2006 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -21,7 +21,7 @@
 
 #include "dvdisaster.h"
 
-#include "rs01-window.h"
+#include "rs01-includes.h"
 
 /***
  *** Forward declarations
@@ -167,7 +167,7 @@ static gboolean set_max_idle_func(gpointer data)
    return FALSE;
 }
 
-void SetFixMaxValues(RS01Widgets *wl, int data_bytes, int ecc_bytes, gint64 sectors)
+void RS01SetFixMaxValues(RS01Widgets *wl, int data_bytes, int ecc_bytes, gint64 sectors)
 {
    wl->dataBytes = data_bytes;
    wl->eccBytes  = ecc_bytes;
@@ -192,7 +192,7 @@ static gboolean results_idle_func(gpointer data)
    return FALSE;
 }
 
-void UpdateFixResults(RS01Widgets *wl, gint64 corrected, gint64 uncorrected)
+void RS01UpdateFixResults(RS01Widgets *wl, gint64 corrected, gint64 uncorrected)
 {
    wl->corrected = corrected;
    wl->uncorrected = uncorrected;
@@ -216,16 +216,11 @@ static gboolean curve_idle_func(gpointer data)
    for(i=wl->lastPercent+1; i<wl->percent; i++)
       wl->fixCurve->ivalue[i] = wl->fixCurve->ivalue[wl->percent];
 
-   /*** If we have not moved on by at least one pixel, skip the rest */
-
-   if(x0 >= x1)
-      return FALSE;
-
-   /*** Resize the Y axes if speed value exceeds current maximum */
+   /*** Resize the Y axes if error values exceeds current maximum */
 
    if(wl->fixCurve->ivalue[wl->percent] > wl->fixCurve->maxY)
    {  wl->fixCurve->maxY = wl->fixCurve->ivalue[wl->percent];
-      wl->fixCurve->maxY = wl->fixCurve->maxY % 5 + 5;
+      wl->fixCurve->maxY = wl->fixCurve->maxY - (wl->fixCurve->maxY % 5) + 5;
 
       update_geometry(wl);
       gdk_window_clear(wl->fixCurve->widget->window);
@@ -241,7 +236,7 @@ static gboolean curve_idle_func(gpointer data)
    {  gdk_gc_set_rgb_fg_color(Closure->drawGC, Closure->red);
       gdk_draw_rectangle(wl->fixCurve->widget->window,
 			 Closure->drawGC, TRUE,
-			 x0, y, x1-x0, wl->fixCurve->bottomY-y);
+			 x0, y, x0==x1 ? 1 : x1-x0, wl->fixCurve->bottomY-y);
    }
    wl->lastPercent = wl->percent;
 
@@ -259,7 +254,7 @@ static gboolean curve_idle_func(gpointer data)
  * Add one new data point 
  */
 
-void AddFixValues(RS01Widgets *wl, int percent, int ecc_max)
+void RS01AddFixValues(RS01Widgets *wl, int percent, int ecc_max)
 {
    if(percent < 0 || percent > 1000)
      return;
@@ -332,7 +327,7 @@ void ResetRS01FixWindow(Method *method)
    gtk_notebook_set_current_page(GTK_NOTEBOOK(wl->fixNotebook), 0);
 
    ZeroCurve(wl->fixCurve);
-   UpdateFixResults(wl, 0, 0);
+   RS01UpdateFixResults(wl, 0, 0);
 
    if(wl->fixCurve && wl->fixCurve->widget)
    {  gdk_window_clear(wl->fixCurve->widget->window);
@@ -443,6 +438,7 @@ static void scale_cb(GtkWidget *widget, gpointer data)
    {  case PREF_CACHE:
 	value = gtk_range_get_value(GTK_RANGE(widget));
         Closure->cacheMB = cache_size[value];
+	UpdateMethodPreferences();
 	break;
       case PREF_NROOTS:
 	value = gtk_range_get_value(GTK_RANGE(widget));
@@ -501,6 +497,16 @@ static void toggle_cb(GtkWidget *widget, gpointer data)
    }
 }
 
+void ResetRS01PrefsPage(Method *method)
+{  RS01Widgets *wl = (RS01Widgets*)method->widgetList;
+   int index;
+
+   for(index = 0; index < sizeof(cache_size)/sizeof(int); index++)
+     if(cache_size[index] > Closure->cacheMB)
+       break;
+
+   gtk_range_set_value(GTK_RANGE(wl->cacheScale), index > 0 ? index-1 : index);
+}
 
 void CreateRS01PrefsPage(Method *method, GtkWidget *parent)
 {  RS01Widgets *wl = (RS01Widgets*)method->widgetList;
@@ -513,7 +519,7 @@ void CreateRS01PrefsPage(Method *method, GtkWidget *parent)
    frame = gtk_frame_new(_utf("Redundancy for new error correction files"));
    gtk_box_pack_start(GTK_BOX(parent), frame, FALSE, FALSE, 0);
 
-   vbox = gtk_vbox_new(FALSE, 20);
+   vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_container_add(GTK_CONTAINER(frame), vbox);
 
@@ -598,14 +604,14 @@ void CreateRS01PrefsPage(Method *method, GtkWidget *parent)
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);
    gtk_container_add(GTK_CONTAINER(frame), hbox);
 
-   lab = gtk_label_new(_("Use"));
+   lab = gtk_label_new(_utf("Use"));
    gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);
 
    for(index = 0; index < sizeof(cache_size)/sizeof(int); index++)
      if(cache_size[index] > Closure->cacheMB)
        break;
 
-   scale = gtk_hscale_new_with_range(0,16,1);
+   scale = wl->cacheScale = gtk_hscale_new_with_range(0,16,1);
    gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_RIGHT);
    gtk_range_set_increments(GTK_RANGE(scale), 1, 1);
    gtk_range_set_value(GTK_RANGE(scale), index > 0 ? index-1 : index);
