@@ -275,8 +275,8 @@ assume_cd:
    cmd[0] = 0x43;  /* READ TOC/PMA/ATIP */
    cmd[2] = 0;     /* format; we want the TOC */
    cmd[6] = 1;     /* track/session number */
-   cmd[7] = 0;     /* allocation length */
-   cmd[8] = length;
+   cmd[7] = (length>>8) & 0xff; /* allocation length */
+   cmd[8] = length & 0xff;
 
    if(SendPacket(dh, cmd, 10, buf, length, &sense, DATA_READ)<0)
    {  Stop(_("%s\nCould not read TOC.\n"),
@@ -321,8 +321,8 @@ assume_cd:
    cmd[1] = 0x02;  /* TIME bit required for this format */
    cmd[2] = 2;     /* format; we want the full TOC */
    cmd[6] = 1;     /* track/session number */
-   cmd[7] = 0;     /* allocation length */
-   cmd[8] = length;
+   cmd[7] = (length>>8) & 0xff; /* allocation length */
+   cmd[8] = length & 0xff;
 
    if(SendPacket(dh, cmd, 10, buf, length, &sense, DATA_READ)<0)
    {  Stop(_("%s\nCould not read full TOC.\n"),
@@ -980,8 +980,30 @@ DeviceHandle* OpenAndQueryDevice(char *device)
 	    device, dh->aspiUsed ? "ASPI" : "SPTI", dh->devinfo);
 #endif
 
+   /* Query the type and fail immediately if incompatible medium is found
+      so that the later tests are not derailed by the wrong medium type */
+
    query_type(dh, 0);
 
+   if(dh->subType == UNSUPPORTED)
+   {  char *td = alloca(strlen(dh->typedescr)+1);
+
+      strcpy(td, dh->typedescr);
+      CloseDevice(dh);
+      Stop(_("This software does not support \"%s\" type media."), td);
+      return NULL;
+   }
+
+   if(dh->sessions>1)
+   {  int sessions = dh->sessions;
+
+      CloseDevice(dh);
+      Stop(_("This software does not support multisession (%d sessions) media."), sessions);
+      return NULL;
+   }
+
+   /* Examine medium type */
+
    if(Closure->querySize >= 1)  /* parseUDF or better requested */
      ExamineUDF(dh);
 
@@ -1035,14 +1057,6 @@ DeviceHandle* OpenAndQueryDevice(char *device)
       Stop(_("This software does not support encrypted media.\n"));
    }
 
-   if(dh->sessions>1)
-   {  int sessions = dh->sessions;
-
-      CloseDevice(dh);
-      Stop(_("This software does not support multisession (%d sessions) media."), sessions);
-      return NULL;
-   }
-
    /* Create the bitmap of simulated defects */
 
    if(Closure->simulateDefects)
